STRICT = true
DEBUG = true

require 'zoetrope'
__ = require 'underscore'
--inspect = require 'inspect'
require 'pepperprof'
require 'getopt_alt'

require 'sprite'
require 'animation'
require 'player'
require 'balloon'

util = {
   dim = function(dir)
      if dir == 'x' then
         return 'width'
      elseif dir == 'y' then
         return 'height'
      else
         if STRICT then error('dir '..dir) end
      end
   end
}

GameView = View:extend {
   onNew = function (self)
              self:loadLayers('data/map.lua')
              self.focus = the.player
              self:clampTo(self.map)

              the.recorder = Recorder:new{mousePosInterval = 9999}
              the.app.meta:add(the.recorder)
              if the.app.record then
                 the.recorder:startRecording()
              elseif the.app.playback then
                 local storage = Storage:new{filename = 'record.lua'}
                 storage:load()
                 --print(inspect(storage.data))
                 the.recorder.record = storage.data
                 the.recorder:startPlaying()
              end
           end,
   onUpdate = function (self)
                 --print('drawTook: ', the.drawTook)
                 --print('tick')
                 --the.player:collide(self.map)
                 --self.map:collide(the.player)
              end,
   -- draw = function (self, x, y)
   --           View.draw(self, x, y)

   --           love.graphics.print('FPS:' .. love.timer.getFPS(), 20, 20)
   --        end
}

the.app = App:new {
   record = true,
   onRun = function (self)
              self.view = GameView:new()
              if DEBUG then
                 self.console:watch('onGround', 'the.player.onGround')
                 self.console:watch('onWall', 'the.player.onWall')
                 self.console:watch('updateTook', 'the.updateTook')
                 self.console:watch('drawTook', 'the.drawTook')
                 self.console:watch('recorder state', 'the.recorder.state')
              end

              --the.profiler = newProfiler('time', 2000)
              --the.profiler = newProfiler()
              --the.profiler:start()
           end,
   onUpdate = function (self, dt)
                 if the.keys:justPressed('escape') then
                    if the.profiler then
                       the.profiler:stop()
                       local outfile = io.open( "profile.txt", "w+" )
                       the.profiler:report( outfile )
                       outfile:close()
                    end

                    if self.record then
                       if not love.filesystem.remove('record.lua') then
                          print('could not remove record.lua')
                       end
                       local storage = Storage:new{
                          data = the.recorder.record,
                          filename = 'record.lua'
                       }
                       storage:save(false)
                       --print(inspect(the.recorder.record))
                    end

                    self.quit()
                 end
              end,
   update = function (self, dt)
               the.updateStart = love.timer.getMicroTime()
               App.update(self, dt)
               if the.updateStart then
                  the.updateTook = love.timer.getMicroTime() - the.updateStart
               end
            end
}

function love.load (arg)
   opts = getopt(arg, '')
   if opts['p'] then
      the.app.playback = true
      the.app.record = false
   elseif opts['r'] then
      the.app.record = true
   end

   the.app:run()
end